package rvl.swv;
import rvl.swv.*;

/**
  This interface specifies the methods needed to buffer code
  and run it in the desired program.
  <p>
  In most cases, the easiest way to implement this interface is to instead
  extend {@link AbstractEngine}, and overriding just a few of its methods.
  <p>If you do implement this interface directly,
  you need to have it call <tt>openCodeFile</tt> and <tt>setParent</tt>
  in the constructor.
 */
public interface EngineInterface {

    /** Open a file for saving the code. Return <tt>true</tt> if successful. */
    public boolean openCodeFile(String baseName);

    /** Close the code file */
    public void closeCodeFile();

    /** Delete the code file */
    public void deleteCodeFile();

    /** Return code and output file names */
    public String[] getFileNames();

    /** Set the parents that drive this instance */
    public void setParents(StatWeave parent, FileInterface filei);

    /** Set where to find the program to run on the code.
    (This need not be called if only tangling)
    */
    public void setBinary(String binloc);

    /** Add a line of code to the code file */
    public void putCode(String line);

    /** Evaluate an expression and put it in the output stream */
    public void putExpr(String expr);

    /** Put startup code in the Code file.
    Test <TT>weaving</TT> to discriminate between startup code for weaving and that for tangling.
    */
    public void putStartup(boolean weaving);


    /** Put code that will add the given text in a separate line of
    the engine's output. This is used by the parent to keep track of
    beginnings and ends of output chunks.
    */
    public void putSeparator(String text);

    /** Put a comment line into the source code. */
    public void putComment(String text);

    /** Put code that will set up a file to receive figure output,
    and return its specs in a <tt>FigFile</tt> object */
    public FigFile setupFig(String chunkName);

    /** Put code to close the graphics file created by <TT>setupFig</TT> */
    public void closeFig();

    /** Run the code, and return the its <TT>Process</TT> object.
    (It is permissible to return <tt>null</tt> to have <tt>StatWeave</tt> skip over it.)
    */
    public Process runCode();


}

