package rvl.swv;
import rvl.swv.*;
import java.util.*;
import java.util.regex.*;

/**
  Implements LaTeX-style syntax for {@link LaTeXFile}.
  In this syntax, code chunks are enclosed in an environment named
  <tt><I>lang</I>code</tt>,
  expressions are of the form <tt><I>\lang</I>expr{<I>expr</I>}</tt>,
  and language-specific options are of the form <tt>\<I>lang</I>weaveOpts{<I>opts</I>}</tt>
 */
public class LaTeXSyntax implements SyntaxInterface {

    /** Pattern for key landmarks */
    Pattern chunkpat, optpat, inlinepat, Optspat, bracepat;

    /** Matcher for current line */
    Matcher matchThisLine;

    /** Flag for whether a line is still being considered */
    boolean matchInProgress = false;

    String thisLine, details, lastMatch, // active strings
        chunkend, codeRef;

    Vector<String> codeRefArgs;         // used for args in code references

    FileInterface filei;
    StatWeave parent;

    int startPos;      // starting popsition used by getOptionList



    /** Default constructor.  Need to call setFileInterface before we can use it */
    public LaTeXSyntax () {
        chunkpat = Pattern.compile("^\\\\begin\\{([a-zA-Z]+)code\\}");
        inlinepat = Pattern.compile("\\\\(recall([a-z]+)|([a-zA-Z]+)expr)\\{([^\\}]+)\\}");
//        exprpat = Pattern.compile("\\\\([a-zA-Z]+)expr\\{");
        Optspat = Pattern.compile("^\\\\([a-zA-Z]+)weaveOpts\\{");
        bracepat = Pattern.compile("\\{([^\\}]*)\\}");  // anything in braces
//        recallpat = Pattern.compile("\\\\recall([a-z]+)\\{([a-zA-Z0-9]+)\\}");
        chunkend = "\\end{Scode}"; // this should be overwritten before it's needed
        codeRef = "\\coderef{";
    }


    /** Alternative constructor */
    public LaTeXSyntax (FileInterface fi) {
        this();
        setFileInterface(fi);
    }

    /**
    Set the <tt>FileInterface</tt> associated with this instance.
    Implementors probably also may want to get parent info from it
    */
    public void setFileInterface (FileInterface filei) {
        this.filei = filei;
        this.parent = filei.getParent();
    }

    /**
    Test a line of the source file.  If it starts a code chunk,
    return its language.  Otherwise, return <tt>null</tt>
    */
    public String testChunk(String line) {
        Matcher mat = chunkpat.matcher(line);
        if (!mat.find()) return null;
        thisLine = line;
        chunkend = "\\end{" + mat.group(1) + "code}";
        startPos = thisLine.indexOf("}"); // next "{" starts options, if any
        return mat.group(1);
    }

    /**
    Return the option list for the code chunk just detected.
    This is required to be a comma-delimited list
    */
    public String getOptionList() {
        int pos = thisLine.indexOf("{", startPos);
        if (pos < 0) return null;
// replaced by next line       String opt = thisLine.substring(pos);
        String opt = thisLine.substring(pos).trim();
        while(!opt.endsWith("}")) {
            String nextLine = filei.getLine();
            if (nextLine == null)
                parent.error("LaTeXSyntax: File ended before options complete",Err.SRCERR);
            opt += nextLine.trim();
        }
        opt = opt.substring(1,opt.length()-1); // strip off the braces
        return opt;
    }

    /**
    Return whether or not the current line ends a code chunk
    */
    public boolean isChunkEnd(String line) {
        return line.startsWith(chunkend);
    }

    /**
    Test a source line to see if it is a global or language-specific option
    specification.  Return <tt>null</tt> if not, the name of the language
    if it is language-specific, or an empty string if it is global.
    The options themselves are retrieved using <tt>getOptionList()</tt>.
    */
    public String testOpts(String line) {
        String lang = null;
        if (line.startsWith("\\weaveOpts{")) lang = "";
        else {
            Matcher mat = Optspat.matcher(line);
            if (mat.find()) lang = mat.group(1);
        }
        if (lang != null) {
            thisLine = line;
            startPos = 0;  // next "{" starts options
        }
        return lang;
    }



    /**
    Test a line of code.  If it is a code reference, return its label,
    else null.  If non-null, any arguments supplied are available via <tt>getArgs()</tt>
    */
    public String testCoderef(String line) {
        if (!line.startsWith(codeRef))
            return null;
        String label = null;
        Matcher mat = bracepat.matcher(line);
        if (!mat.find())
            parent.error("Syntax error in \\Coderef statement", Err.SRCERR);
      // We have a code ref -- get the info
        label = mat.group(1);
        codeRefArgs = new Vector<String>(3);
        while(mat.find())
            codeRefArgs.addElement(mat.group(1));
        return label;
    }


    /** Return the vector of arguments in latest code reference */
    public Vector<String> getArgs() {
        return codeRefArgs.size() > 0 ? codeRefArgs : null;
    }


    /** Test the line for an occurrence of an expression or a recall,
    and return the appropriate tag.  The <tt>line</tt> argument matters
    only on the first call for this line.  Subsequent calls find subsequent
    matches.  A null is return when there are no more */
    public Tag testInline(String line) {
        if (!matchInProgress) {
            thisLine = line;
            matchThisLine = inlinepat.matcher(line);
            matchInProgress = true;
        }
        if (!matchThisLine.find()) {
            matchInProgress = false;
            return null;
        }
        lastMatch = matchThisLine.group();
        if (matchThisLine.group(2) == null) { // we have an expression
            details = matchThisLine.group(3) + " " + matchThisLine.group(4);
            return Tag.expr;
        }
        else { // we have a recall directive
            String type = matchThisLine.group(2);
            Tag tag = type.equals("fig") ? Tag.recallFig
                : type.equals("code") ? Tag.recallCode
                : type.equals("out") || type.equals("output") ? Tag.recallOutput
                : Tag.noMore;
            if (tag == Tag.noMore)
                parent.error("Invalid recall in " + filei.getPosition() + ": "
                    + lastMatch,    Err.SRCERR);
            details = matchThisLine.group(4);
            return tag;
        }
    }

    /** Return details of last result of <tt>testInline()</tt>.
    If it is an expression, it is <tt>"lang expr"</tt>,
    otherwise for a recall, it is the label being recalled. */
    public String getDetails() {
        return details;
    }

    /** Return exact string that was matched by most recent tag search */
    public String getLastMatch() {
        return lastMatch;
    }


} // END of LaTeXSyntax