package rvl.swv;
import rvl.swv.*;
import java.util.*;

/**
 * This interface specifies methods needed to detect code chunks
 * and other weaving-related patterns in lines of the source file.
 * This is intended for use with file formats that are markup-based
 * (like LaTeX), to allow the possibility of providing a different
 * markup syntax for the same file format.
 *
 * @author Russell V. Lenth
 * @version May 9, 2008
 */
public interface SyntaxInterface {

    /**
    Set the <tt>FileInterface</tt> associated with this instance.
    Implementors probably also may want to get parent info from it
    */
    public void setFileInterface (FileInterface filei);

    /**
    Test a line of the source file.  If it starts a code chunk,
    return its language.  Otherwise, return <tt>null</tt>
    */
    public String testChunk(String line);

    /**
    Return the option list for the code chunk just detected.
    This is required to be a comma-delimited list
    */
    public String getOptionList();

    /**
    Return whether or not the current line ends a code chunk
    */
    public boolean isChunkEnd(String line);


    /**
    Test a source line to see if it is a global or language-specific option
    specification.  Return <tt>null</tt> if not, the name of the language
    if it is language-specific, or an empty string if it is global.
    The options themselves are retrieved using <tt>getOptionList()</tt>.
    */
    public String testOpts(String line);



    /**
    Test a line of code.  If it is a code reference, return its label,
    else null.  If non-null, any arguments supplied are available via <tt>getArgs()</tt>
    If it is a "starred" code reference, precede the returned label with a <tt>*</tt>.
    */
    public String testCoderef(String line);


    /** Return the vector of arguments in latest code reference */
    public Vector<String> getArgs();


    /** Test the line for an occurrence of an expression or a recall,
    and return the appropriate tag.  The <tt>line</tt> argument matters
    only on the first call for this line.  Subsequent calls find subsequent
    matches.  When there are no more, a <tt>null</tt> should be returned. */
    public Tag testInline(String line);

    /** Return details of last result of <tt>testInline()</tt>.
    If it is an expression, it is <tt>"lang expr"</tt>,
    otherwise for a recall, it is the label being recalled. */
    public String getDetails();

    /** Return exact string that was matched by most recent tag search */
    public String getLastMatch();

}